"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var history_1 = require("history");
var history_2 = require("./history");
var web_1 = require("@redux-model/web");
var ForgetRegisterError_1 = require("@redux-model/web/core/exceptions/ForgetRegisterError");
var RouterModel = /** @class */ (function (_super) {
    tslib_1.__extends(RouterModel, _super);
    function RouterModel() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.pathListeners = [];
        _this.changeHistory = _this.actionNormal(function (_, payload) {
            return payload;
        });
        _this.actionPush = _this.actionNormal(function () {
            return {
                location: _this.getHistory().location,
                action: _this.getHistory().action,
            };
        });
        _this.actionReplace = _this.actionNormal(function () {
            return {
                location: _this.getHistory().location,
                action: _this.getHistory().action,
            };
        });
        _this.actionGo = _this.actionNormal(function () {
            return {
                location: _this.getHistory().location,
                action: _this.getHistory().action,
            };
        });
        _this.actionGoBack = _this.actionNormal(function () {
            return {
                location: _this.getHistory().location,
                action: _this.getHistory().action,
            };
        });
        _this.actionGoForward = _this.actionNormal(function () {
            return {
                location: _this.getHistory().location,
                action: _this.getHistory().action,
            };
        });
        return _this;
    }
    RouterModel.prototype.push = function (path, state) {
        this.getHistory().push(path, state);
    };
    RouterModel.prototype.replace = function (path, state) {
        this.getHistory().replace(path, state);
    };
    RouterModel.prototype.go = function (index) {
        this.getHistory().go(index);
    };
    RouterModel.prototype.goBack = function () {
        this.getHistory().goBack();
    };
    RouterModel.prototype.goForward = function () {
        this.getHistory().goForward();
    };
    RouterModel.prototype.listen = function (path, fn) {
        this.pathListeners.push([path, fn]);
    };
    RouterModel.prototype.registerBrowser = function (history) {
        var originalHistory = history_2.getHistory();
        var newHistory = history || originalHistory || history_1.createBrowserHistory();
        history_2.setHistory(newHistory);
        if (originalHistory && originalHistory !== newHistory && this.unregister) {
            this.unregister();
        }
        return this.register();
    };
    RouterModel.prototype.registerHash = function (history) {
        var originalHistory = history_2.getHistory();
        var newHistory = history || originalHistory || history_1.createHashHistory();
        history_2.setHistory(newHistory);
        if (originalHistory && originalHistory !== newHistory && this.unregister) {
            this.unregister();
        }
        return this.register();
    };
    RouterModel.prototype.register = function () {
        var history = history_2.getHistory();
        if (!history) {
            throw new ReferenceError('Use "registerBrowser()" or "registerHash()" for routerModel.');
        }
        if (!this.unregister) {
            this.unregister = history.listen(this.onHistoryChange.bind(this));
        }
        return _super.prototype.register.call(this);
    };
    RouterModel.prototype.onHistoryChange = function (location, action) {
        this.changeHistory({
            location: location,
            action: action,
        });
    };
    RouterModel.prototype.getHistory = function () {
        var history = history_2.getHistory();
        if (!history) {
            throw new ForgetRegisterError_1.ForgetRegisterError('RouterModel');
        }
        return history;
    };
    RouterModel.prototype.initReducer = function () {
        var _this = this;
        return function () {
            return {
                location: _this.getHistory().location,
                action: _this.getHistory().action,
            };
        };
    };
    return RouterModel;
}(web_1.Model));
exports.routerModel = new RouterModel();
